"use strict";
var domain_1 = require('../../serenity/domain');
var file_system_1 = require('../../serenity/io/file_system');
var serenity_1 = require('../../serenity/serenity');
var abilities_1 = require('../screenplay/abilities');
var md5_1 = require('ts-md5/dist/md5');
exports.TakeAPicture = {
    Before_Activity: [domain_1.ActivityStarts],
    After_Activity: [domain_1.ActivityFinished],
    Before_And_After_Activity: [domain_1.ActivityStarts, domain_1.ActivityFinished],
};
function photographer(eventsOfInterest, pathToReports) {
    if (eventsOfInterest === void 0) { eventsOfInterest = exports.TakeAPicture.After_Activity; }
    if (pathToReports === void 0) { pathToReports = serenity_1.Default_Path_To_Reports; }
    return new Photographer(eventsOfInterest, new file_system_1.FileSystem(pathToReports));
}
exports.photographer = photographer;
var Photographer = (function () {
    function Photographer(eventsOfInterest, fs, naming) {
        if (naming === void 0) { naming = new Md5HashedPictureNames('png'); }
        this.fs = fs;
        this.naming = naming;
        this.eventsOfInterest = [];
        this.eventsOfInterest = this.eventsOfInterest.concat(eventsOfInterest);
    }
    Photographer.prototype.assignTo = function (stage) {
        this.stage = stage;
        this.stage.manager.registerInterestIn(this.eventsOfInterest, this);
    };
    Photographer.prototype.notifyOf = function (event) {
        switch (event.constructor.name) {
            case domain_1.ActivityStarts.name:
                this.whenActivityStarts(event);
                break;
            case domain_1.ActivityFinished.name:
                this.whenActivityFinished(event);
                break;
            default: break;
        }
    };
    Photographer.prototype.whenActivityStarts = function (event) {
        if (!!this.stage && this.stage.theShowHasStarted()) {
            this.takeAPhotoAndSendAReceipt(event.value, event.timestamp);
        }
    };
    Photographer.prototype.whenActivityFinished = function (event) {
        var skip = domain_1.Result.COMPROMISED | domain_1.Result.ERROR | domain_1.Result.FAILURE | domain_1.Result.SUCCESS;
        if (event.value.result & skip) {
            this.takeAPhotoAndSendAReceipt(event.value.subject, event.timestamp);
        }
    };
    // todo: verify the Significance
    Photographer.prototype.takeAPhotoAndSendAReceipt = function (subject, timestamp) {
        var promisedPicture = this.photographWorkOf(this.stage.theActorInTheSpotlight());
        this.stage.manager.notifyOf(new domain_1.PhotoAttempted(new domain_1.PhotoReceipt(subject, promisedPicture), timestamp));
    };
    Photographer.prototype.photographWorkOf = function (actor) {
        var _this = this;
        var saveScreenshot = function (data) { return _this.fs.store(_this.naming.nameFor(data), new Buffer(data, 'base64')); };
        var ignoreInactiveBrowserButReportAnyOther = function (error) {
            if (error.message.indexOf('does not have a valid session ID') > -1) {
                return undefined;
            }
            throw error;
        };
        return abilities_1.BrowseTheWeb.as(actor).takeScreenshot()
            .then(saveScreenshot)
            .then(function (path) { return new domain_1.Photo(path); }, function (error) { return ignoreInactiveBrowserButReportAnyOther(error); });
    };
    return Photographer;
}());
exports.Photographer = Photographer;
var Md5HashedPictureNames = (function () {
    function Md5HashedPictureNames(fileExtension) {
        if (fileExtension === void 0) { fileExtension = ''; }
        this.fileExtension = fileExtension;
    }
    Md5HashedPictureNames.prototype.nameFor = function (base64encodedData) {
        return md5_1.Md5.hashStr(base64encodedData) + this.extension();
    };
    Md5HashedPictureNames.prototype.extension = function () {
        return !!this.fileExtension ? '.' + this.fileExtension : '';
    };
    return Md5HashedPictureNames;
}());
exports.Md5HashedPictureNames = Md5HashedPictureNames;

//# sourceMappingURL=photographer.js.map
