"use strict";
var domain_1 = require('../domain');
var serenity_1 = require('../serenity');
var step_description_1 = require('./step_description');
var _ = require('lodash');
// todo: add Significance to the @step
var Step = (function () {
    function Step(stageManager) {
        this.stageManager = stageManager;
    }
    Step.prototype.describedUsing = function (template) {
        var description = new step_description_1.StepDescription(template), decorated = this;
        return function (target, propertyKey, descriptor) {
            var performAs = descriptor.value, decorator = _.cloneDeep(descriptor);
            decorator.value = function () {
                var _this = this;
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i - 0] = arguments[_i];
                }
                var activity = description.interpolateWith(this, args);
                return Promise.resolve()
                    .then(function () { return decorated.beforeStep(activity); })
                    .then(function () { return performAs.apply(_this, args); })
                    .then(function () { return decorated.afterStep(activity); }, function (e) { return decorated.onFailure(activity, e); });
            };
            return decorator;
        };
    };
    Step.prototype.beforeStep = function (activity) {
        this.stageManager.notifyOf(new domain_1.ActivityStarts(activity));
    };
    Step.prototype.afterStep = function (activity) {
        this.stageManager.notifyOf(new domain_1.ActivityFinished(new domain_1.Outcome(activity, domain_1.Result.SUCCESS)));
    };
    Step.prototype.onFailure = function (activity, error) {
        // todo: sniff the exception to find out about the Result. Did the test fail, or was it compromised?
        this.stageManager.notifyOf(new domain_1.ActivityFinished(new domain_1.Outcome(activity, domain_1.Result.ERROR, error)));
    };
    return Step;
}());
exports.Step = Step;
function step(stepDescriptionTemplate) {
    return new Step(serenity_1.Serenity.stageManager()).describedUsing(stepDescriptionTemplate);
}
exports.step = step;

//# sourceMappingURL=step_annotation.js.map
